// Generated by dts-bundle v0.7.3-fork.1
// Dependencies for this module:
//   ../../../../../@fullcalendar/core

declare module '@fullcalendar/rrule' {
    const _default: import("@fullcalendar/core").PluginDef;
    export default _default;
}

