.fc-year-container .fc-dayGridWeek-view .fc-content-skeleton,
.fc-year-container .fc-dayGridDay-view .fc-content-skeleton {
    padding-bottom: 1em;
}
.fc-year-container .fc-dayGrid-view .fc-body .fc-row {
    min-height: 5em;
}
.fc-year-container .fc-row.fc-rigid {
    overflow: hidden;
}
.fc-year-container .fc-row.fc-rigid .fc-content-skeleton {
    position: absolute;
    top: 0;
    left: 0;
    right: 0;
}
.fc-year-container .fc-day-top.fc-other-month {
    opacity: 0.3;

}
.fc-year-container .fc-day-top {
    font-size: 10px;

}
.fc-year-container .fc-dayGrid-view .fc-week-number,
.fc-year-container .fc-dayGrid-view .fc-day-number {
    padding: 2px;
}
.fc-year-container .fc-dayGrid-view th.fc-week-number,
.fc-year-container .fc-dayGrid-view th.fc-day-number {
    padding: 0 2px;
}
.fc-year-container .fc-ltr .fc-dayGrid-view .fc-day-top .fc-day-number {
    float: right;
}
.fc-year-container .fc-rtl .fc-dayGrid-view .fc-day-top .fc-day-number {
    float: left;
}
.fc-year-container .fc-ltr .fc-dayGrid-view .fc-day-top .fc-week-number {
    float: left;
    border-radius: 0 0 3px 0;
}
.fc-year-container .fc-rtl .fc-dayGrid-view .fc-day-top .fc-week-number {
    float: right;
    border-radius: 0 0 0 3px;
}
.fc-year-container .fc-dayGrid-view .fc-day-top .fc-week-number {
    min-width: 1.5em;
    text-align: center;
    background-color: #f2f2f2;
    color: #808080;
}
.fc-year-container .fc-dayGrid-view td.fc-week-number {
    text-align: center;
}
.fc-year-container .fc-dayGrid-view td.fc-week-number > * {
    display: inline-block;
    min-width: 1.25em;
}
.fc-year-container .fc-row-monthname {
    height: 100%;
    width: 1.25em;
    position: absolute;
    text-align: center;
    border-right-width: 1px;
    border-right-style: solid;
    border-bottom-width: 1px;
    border-bottom-style: solid;
}
.fc-year-container .fc-row-monthname span {
    display: block;
    transform: rotate(270deg);
    position: relative;
    top: 50%;
    font-size: 10px;
    font-weight: bold;
}
.fc-year-container .fc-bg,
.fc-year-container .fc-content-skeleton {
    left: 1.25em !important;
}
.fc-year-container .fc-day-number-weekday {
    padding-right: 0.3em;
    color: #aaa;
}
