# Generated by Django 3.2.7 on 2023-11-08 09:26

from django.db import migrations, models
import django.db.models.deletion
import time_stamped_model.models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0012_auto_20230718_1642'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReportQueryOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', time_stamped_model.models.CreationDateTimeField(auto_now_add=True)),
                ('modified', time_stamped_model.models.ModificationDateTimeField(auto_now=True)),
                ('order_by_field', models.CharField(blank=True, max_length=200, null=True)),
                ('order_by_ascending', models.BooleanField(default=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('report_query', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='advanced_report_builder.reportquery')),
            ],
            options={
                'ordering': ['order'],
            },
        ),
    ]
