# Generated by Django 3.2.7 on 2024-07-06 22:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0013_reportqueryorder'),
    ]

    operations = [
        migrations.AddField(
            model_name='dashboardreport',
            name='report_query',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='advanced_report_builder.reportquery'),
        ),
        migrations.AddField(
            model_name='dashboardreport',
            name='show_versions',
            field=models.BooleanField(default=True),
        ),
    ]
