// Generated by dts-bundle v0.7.3-fork.1
// Dependencies for this module:
//   ../../../../../moment-timezone/builds/moment-timezone-with-data
//   ../../../../../@fullcalendar/core

declare module '@fullcalendar/moment-timezone' {
    import 'moment-timezone/builds/moment-timezone-with-data';
    const _default: import("@fullcalendar/core").PluginDef;
    export default _default;
}

