# Generated by Django 5.0.7 on 2025-05-29 22:03

import django.db.models.deletion
import time_stamped_model.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0015_report_user_created_report_user_updated'),
    ]

    operations = [
        migrations.CreateModel(
            name='CalendarReport',
            fields=[
                ('report_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='advanced_report_builder.report')),
                ('height', models.PositiveSmallIntegerField(default=600)),
                ('view_type', models.PositiveSmallIntegerField(choices=[(1, 'Month'), (2, 'Grid Week'), (3, 'List Week'), (4, 'Day'), (5, 'Year')], default=1)),
            ],
            options={
                'abstract': False,
            },
            bases=('advanced_report_builder.report',),
        ),
        migrations.AddField(
            model_name='dashboardreport',
            name='options',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name='CalendarReportDescription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', time_stamped_model.models.CreationDateTimeField(auto_now_add=True)),
                ('modified', time_stamped_model.models.ModificationDateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(blank=True, null=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('calendar_report', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='advanced_report_builder.calendarreport')),
                ('report_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='advanced_report_builder.reporttype')),
            ],
            options={
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='CalendarReportDataSet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', time_stamped_model.models.CreationDateTimeField(auto_now_add=True)),
                ('modified', time_stamped_model.models.ModificationDateTimeField(auto_now=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('heading_field', models.CharField(blank=True, max_length=200, null=True)),
                ('name', models.CharField(max_length=200)),
                ('display_type', models.PositiveSmallIntegerField(choices=[(1, 'Name and Description'), (2, 'Description Only'), (3, 'Heading Only')], default=1)),
                ('start_date_field', models.CharField(blank=True, max_length=200, null=True)),
                ('end_date_type', models.PositiveSmallIntegerField(choices=[(1, 'Field'), (2, 'Duration Field'), (3, 'Duration Fixed')], default=1)),
                ('end_date_field', models.CharField(blank=True, max_length=200, null=True)),
                ('end_duration_field', models.CharField(blank=True, max_length=200, null=True)),
                ('end_duration', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('background_colour_field', models.CharField(blank=True, max_length=200, null=True)),
                ('link_field', models.CharField(blank=True, max_length=200, null=True)),
                ('query_data', models.JSONField(blank=True, null=True)),
                ('calendar_report', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='advanced_report_builder.calendarreport')),
                ('report_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='advanced_report_builder.reporttype')),
                ('calendar_report_description', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='advanced_report_builder.calendarreportdescription')),
            ],
            options={
                'ordering': ('order',),
            },
        ),
    ]
