# Generated by Django 5.0.7 on 2025-06-17 13:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0016_calendarreport_dashboardreport_options_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='barchartreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='calendarreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='customreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='dashboard',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='funnelchartreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='kanbanreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='linechartreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='piechartreport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='singlevaluereport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='tablereport',
            options={'get_latest_by': 'created'},
        ),
        migrations.AlterModelOptions(
            name='target',
            options={'get_latest_by': 'created'},
        ),
    ]
