# Generated by Django 3.2.7 on 2022-02-09 16:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='singlevaluereport',
            name='breakdown_fields',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='singlevaluereport',
            name='show_breakdown',
            field=models.BooleanField(default=False),
        ),
    ]
