{% extends 'django_modals/modal_base.html' %}

{% block javascript %}
<script>

$(function () {
        $('#id_report_type').on('change', function (e) {
            $('#id_order_by_field').val('').trigger('change');
        });
    });

</script>
{% endblock %}