{% extends 'django_modals/modal_base.html' %}

{% block javascript %}
<script>
            $(function () {
                $('#id_report_type').on('change', function (e) {
                    $('#id_field').val('').trigger('change');
                    $('#id_numerator').val('').trigger('change');
                });
            });

</script>
{% endblock %}